// Set up the SVG
const width = 1600;
const height = 1000;
const svg = d3
  .select('#network-graph')
  .attr('width', width)
  .attr('height', height);

// Marker for arrowheads
svg
  .append('defs')
  .selectAll('marker')
  .data(['end'])
  .enter()
  .append('marker')
  .attr('id', String)
  .attr('viewBox', '0 -5 10 10')
  .attr('refX', 25)
  .attr('refY', 0)
  .attr('markerWidth', 4)
  .attr('markerHeight', 4)
  .attr('orient', 'auto')
  .append('path')
  .attr('d', 'M0,-5L10,0L0,5');

// Add title
svg
  .append('text')
  .attr('x', width / 2)
  .attr('y', 40)
  .attr('text-anchor', 'middle')
  .style('font-size', '30px')
  .style('font-family', 'Arial, sans-serif')
  .style('font-weight', 'bold')
  .text('Top Product Categories and Countries Network');

// Custom color palette for nodes
const categoryColor = '#66c2a5'; // Custom color for categories
const countryColor = '#fc8d62'; // Custom color for countries

// Define the URL for fetching the data
const csvUrl =
  'https://raw.githubusercontent.com/Tagore1427/Project_data_DV/main/Global_Superstore2.csv';

// Fetch the CSV data
d3.csv(csvUrl).then((data) => {
  let categoryCounts = {};
  let countryCounts = {};
  data.forEach((d) => {
    categoryCounts[d.Category] =
      (categoryCounts[d.Category] || 0) + 1;
    countryCounts[d.Country] =
      (countryCounts[d.Country] || 0) + 1;
  });

  // Sort and take top N
  let topCategories = Object.keys(categoryCounts)
    .sort((a, b) => categoryCounts[b] - categoryCounts[a])
    .slice(0, 20);
  let topCountries = Object.keys(countryCounts)
    .sort((a, b) => countryCounts[b] - countryCounts[a])
    .slice(0, 20);

  // Filter data
  const filteredData = data.filter(
    (d) =>
      topCategories.includes(d.Category) &&
      topCountries.includes(d.Country),
  );

  // Nodes and Links
  const nodes = [
    ...new Set(
      filteredData
        .map((d) => d.Category)
        .concat(filteredData.map((d) => d.Country)),
    ),
  ].map((id) => ({
    id,
    group: topCategories.includes(id) ? 1 : 2,
  }));

  const links = filteredData.map((d) => ({
    source: d.Category,
    target: d.Country,
  }));

  // Create the simulation
  const simulation = d3
    .forceSimulation(nodes)
    .force(
      'link',
      d3
        .forceLink(links)
        .id((d) => d.id)
        .distance(250),
    )
    .force('charge', d3.forceManyBody().strength(-300))
    .force('center', d3.forceCenter(width / 2, height / 2));

  // Draw the links with opacity
  const link = svg
    .append('g')
    .attr('class', 'links')
    .selectAll('line')
    .data(links)
    .enter()
    .append('line')
    .attr('stroke', '#bbb')
    .attr('stroke-opacity', 0.6)
    .attr('stroke-width', 1.5)
    .attr('marker-end', 'url(#end)');

  // Draw the nodes with outlines and custom colors
  const node = svg
    .append('g')
    .attr('class', 'nodes')
    .selectAll('circle')
    .data(nodes)
    .enter()
    .append('circle')
    .attr('r', 10)
    .attr('fill', (d) =>
      d.group === 1 ? categoryColor : countryColor,
    )
    .style('stroke', '#fff')
    .style('stroke-width', '1.5px')
    .call(
      d3
        .drag()
        .on('start', dragstarted)
        .on('drag', dragged)
        .on('end', dragended),
    );

  // Add labels for the nodes
  const label = svg
    .append('g')
    .attr('class', 'labels')
    .selectAll('text')
    .data(nodes)
    .enter()
    .append('text')
    .attr('x', 14)
    .attr('y', '.31em')
    .style('font-size', '12px')
    .style('font-family', 'Arial, sans-serif')
    .text((d) => d.id);

  // Add tick instructions for positioning
  simulation.on('tick', () => {
    link
      .attr('x1', (d) => d.source.x)
      .attr('y1', (d) => d.source.y)
      .attr('x2', (d) => d.target.x)
      .attr('y2', (d) => d.target.y);

    node.attr('cx', (d) => d.x).attr('cy', (d) => d.y);

    label.attr('x', (d) => d.x + 5).attr('y', (d) => d.y);
  });

  // Drag functions for interaction
  function dragstarted(event) {
    if (!event.active)
      simulation.alphaTarget(0.3).restart();
    event.subject.fx = event.subject.x;
    event.subject.fy = event.subject.y;
  }

  function dragged(event) {
    event.subject.fx = event.x;
    event.subject.fy = event.y;
  }

  function dragended(event) {
    if (!event.active) simulation.alphaTarget(0);
    event.subject.fx = null;
    event.subject.fy = null;
  }
});
